C============================================================================
C package Decode
C         ~~~~~~
C
C DESCRIPTION: Contains routines to decode PIC and BIC files which have been
C              uploaded to the VAX from a PC.
C
C              This code is cloned from the conversion program PICTOMOR.FOR
C
C============================================================================

C============================================================================
	subroutine Decode_PIC (OK, fin, numCols4, numRows4, im)
C
C DESCRIPTION: Opens the input file; reads number of columns/rows, then
C              places the decoded pixel data in an array.
C
C INPUT: PICName - Name of inpout file. The .PIC file should be 
C                  run-length encoded according to the HALO
C                  format for .CUT files. When the .PIC is uploaded to 
C                  the VAX, it must be transferred as 'binary' data, so 
C                  that no funny control characters such as start-of-record, 
C                  end-of-record, etc. get inserted.
C        
C OUTPUT: OK       - true if file opened, read, and closed OK.
C         numCols4, 
C         numRows4 - size of output pixel data
C         im       -  The output array will be of 2-byte integers. 
C                     The array is indexed: (Column_index, Row_index)
C
C COMPILATION: fortran Decode
C
C AUTHOR: Mark Tooker
C         Geological Survey of Canada
C 
C DATE: June 12, 1989
C
C=============================================================================

C=========== Program storage =================================================

C---------- subroutine parameters -----------------------------------

        integer*4 NumCols4, NumRows4   ! size of output data file
        integer*2 im(4096,4096)        ! output pixel data
        logical   OK                   ! true if open, read, close OK
        character fin*32               ! input name of PIC file

C---------- utility storage -------------------------------------------

	integer*2 NumBytes, SaveNum    ! record # of bytes of code for 1 line

        character*512 Buff             ! holds 512 bytes of input data

	character Flag, DataByte       ! hold 1 byte from Buff

        integer*4 ByteCount            ! pointer into Buff for Transfer

	character ScratchChar2*2       ! 2-bytes that can converted to integer*2

	data InUnit /2/                ! logical unit numbers of files 

C=========== Main body of subroutine Decode_PIC =============================

	open (unit=InUnit,file=fin,status='OLD',form='UNFORMATTED',
     +        err=999)

C...Read the first 512 bytes of .PIC file.....

        read (InUnit) Buff
       
C...Get the number of rows, columns.....

	ByteCount = 1
	call Transfer2 (ScratchChar2, Buff, ByteCount, InUnit)
        NumCols4 = ichar(ScratchChar2(2:2))*256+ichar(ScratchChar2(1:1))

        call Transfer2 (ScratchChar2, Buff, ByteCount, InUnit)
        NumRows4 = ichar(ScratchChar2(2:2))*256+ichar(ScratchChar2(1:1))

C        print *, ' '
C	print *, 'Number of rows = ',NumRows4, '. Number of columns =',NumCols4
C
C       write (OutUnit) NumRows4, NumCols4

C...Read the data, convert from runlength-encoded to straight pixels, & write...
C-------------------------------------------------------------------------------

C...Set up first line record's byte offset in buffer.....

	ByteCount = 7

C.......For each RL-encoded line, get # bytes.....

        do i = 1, NumRows4

C..........Get the # encoded bytes record.....

           call Transfer2 (ScratchChar2, Buff, ByteCount, InUnit)

                                                ! the 2-byte result must be
                                                ! converted to a NATURAL (>=0)
                                                ! integer value

           NumBytes = ichar(ScratchChar2(2:2)) * 256        
     +                + ichar(ScratchChar2(1:1))
	   SaveNum = NumBytes

C..........Loop from label 100 until entire line read/written.....

	   j = 1                          ! initialize column counter
100        continue

	      call Transfer1 (Flag, Buff, ByteCount, InUnit)   ! get code flag 
              NumBytes = NumBytes - 1

C.............see if Flag byte is (1nnn nnnn, 0nnn nnnn, or n000 0000).....

              if (ichar(Flag) .gt. 128) then        ! (1nnn nnnn) - dupl. data
                 NumDupl = ichar(Flag) - 128        ! ........................

	         call Transfer1 (DataByte, Buff, ByteCount, InUnit) ! get data
	         NumBytes = NumBytes - 1

                 do j = j, (j + NumDupl - 1)        ! write data NumDupl times
	            im (j,i) = ichar(DataByte)
	         end do

                 goto 100
                                                    ! (0nnn nnnn) - unique data
                                                    ! .........................

              else if ((ichar(Flag).lt.128).and.(ichar(Flag).gt.0)) then
                 do j = j, (j + ichar(Flag) - 1)
                                                    ! read/write each byte
	            call Transfer1 (DataByte, Buff, ByteCount, InUnit)
		    NumBytes = NumBytes - 1

		    im (j,i) = ichar(DataByte)
                 end do

                 goto 100
           
	      else                                 ! (n000 0000) - end of line
                                                   ! .........................

C...Check if a full line of decoded pixels has been produced.....

	         if (j .ne. (NumCols4 + 1)) then
		    print *, ' '
		    print *,'DECODE ERROR: # line data bytes <> # columns.',
     +                       'ROW = ', i, '. Should be:',NumCols4,
     +                       ', actual:',j + 1,'.'
                    print *, 'Continuing decoding.'
                 end if

C...Check if the number of encoded bytes was correct.....

                 if (NumBytes .ne. 0) then
		    print *,' '
		    print *, 'DECODE ERROR: ROW=',i,': # of encoded bytes = ',
     +                       (SaveNum - NumBytes), ', should be =', SaveNum
		    print *, 'Continuing decoding.'
		 end if

C...write the line of decoded pixels, restore pixel-counter...

                 j = 1

              end if
	end do

C...normal completion of subroutine.....

	close (InUnit)
        OK = .true.

        return

C...abnormal completion of subroutine.....

999	continue
	OK = .false.

        return
	end

C====== End of subroutine Decode_PIC ====================================

C============================================================================
	subroutine Decode_BIC (OK, BicName, PalName, MaxColors)
C                  ~~~~~~~~~~
C
C DESCRIPTION: Opens the input file; reads palette information;
C              creates new ASCII file with palette.
C
C INPUT: 
C        BICName - Name of input file. The .BIC file contains palette
C                  information according to the SPANS 4.0 format.
C                  When the .BIC is uploaded to the VAX, it must be 
C                  transferred as 'binary' data, so that no funny control 
C                  characters such as start-of-record, end-of-record, 
C                  etc. get inserted.
C
C        PALName - Name of the ASCII .PAL file to be created.
C        
C        MaxColors - the maximum number of RGB color entries to be processed
C
C OUTPUT: 
C        OK     - true if file opened, read, and closed OK.
C
C=============================================================================

C=========== Program storage =================================================

C---------- subroutine parameters -----------------------------------

        logical   OK                       ! true if open, read, close OK
        character BICName*32, PALName*32   ! input name of PIC file
	integer*4 MaxColors                ! max. number to read/write

C---------- utility storage -------------------------------------------

        character*512 Buff             ! holds 512 bytes of input data

        character DataByte             ! holds 1 byte transferred from Buff

        integer*4 ByteCount            ! pointer into Buff for Transfer

        integer*4 RVal, GVal, BVal     ! hold three color values

        data InUnit  /2/               ! file logical unit numbers
        data OutUnit /3/               !

C========= begin subroutine Decode_BIC =======================================

	open (unit=InUnit,file=BICName,status='OLD',
     +        form='UNFORMATTED',err=9991)

        open (unit=OutUnit,file=PALName,form='FORMATTED',
     +        status='NEW',err=9990)

C...Read the first 512 bytes of .PIC file.....

        read (InUnit) Buff

C...loop to read triplets of color info.....

	ByteCount = 151        ! position pointer at first triplet

        do i = 1, MaxColors

           call Transfer1 (DataByte, Buff, ByteCount, InUnit)
           RVal = ichar(DataByte)

           call Transfer1 (DataByte, Buff, ByteCount, InUnit)
           GVal = ichar(DataByte)

           call Transfer1 (DataByte, Buff, ByteCount, InUnit)
           BVal = ichar(DataByte)

           write (OutUnit, 10) i, RVal, GVal, BVal
10	   format ( i4, '     ', i4, '     ', i4, '     ', i4)

	end do

C...normal completion of subroutine.....

900	continue
        close (InUnit)
        close (OutUnit)

        OK = .true.

        return

C...abnormal completion of subroutine.....

9990    continue
        print *,'DECODE_BIC: Error opening ', PalName
        goto 9999

9991	continue
        PRINT *, 'DECODE_BIC: Error opening ', BicName

9999    continue
	OK = .false.
        return
        end

C========= end of subroutine Decode_BIC ======================================

C============================================================================
	subroutine Decode_BSQ (OK, fin, numCols4, numRows4, im)
C
C DESCRIPTION: Opens the input file; reads number of columns/rows, then
C              places the decoded pixel data in an array.
C
C INPUT: PICName - Name of input file. The .PIP file should be 
C                  run-length encoded according to the PCI BSQ
C                  format. When the file is uploaded to 
C                  the VAX, it must be transferred as 'binary' data, so 
C                  that no funny control characters such as start-of-record, 
C                  end-of-record, etc. get inserted.
C
C OUTPUT: OK       - true if file opened, read, and closed OK.
C         numCols4, 
C         numRows4 - size of output pixel data
C         im       -  The output array will be of 2-byte integers. 
C                     The array is indexed: (Column_index, Row_index)
C
C COMPILATION: fortran Decode
C
C AUTHOR: Mark Tooker
C         Geological Survey of Canada
C 
C DATE: July 21, 1989
C
C=============================================================================

C=========== Program storage =================================================

C---------- subroutine parameters -----------------------------------

        integer*4 NumCols4, NumRows4   ! size of output data file
        integer*2 im(4096,4096)        ! output pixel data
        logical   OK                   ! true if open, read, close OK
        character fin*32               ! input name of PIC file

C---------- utility storage -------------------------------------------

        character*512 Buff             ! holds 512 bytes of input data

	character DataByte             ! hold 1 byte from Buff

        integer*4 ByteCount            ! pointer into Buff for Transfer

	character ScratchChar1         ! holds 1 input byte
        character ScratchChar2*2       ! 2-bytes that can converted to integer*2

	data InUnit /2/                ! logical unit numbers of files 

C=========== Main body of subroutine Decode_BSQ =============================

	open (unit=InUnit,file=fin,status='OLD',form='UNFORMATTED',
     +        err=999)

C...Read the first 512 bytes of .PIC file.....

        read (InUnit) Buff
       
C...Get the number of rows, columns.....
C...they are 4-byte unsigned integers.....

	ByteCount = 1

	call Transfer2 (ScratchChar2, Buff, ByteCount, InUnit)
        NumCols4 = ichar(ScratchChar2(2:2))*256+ichar(ScratchChar2(1:1))
	call Transfer2 (ScratchChar2, Buff, ByteCount, InUnit)
        NumCols4 = NumCols4 + 
     +             ichar(ScratchChar2(2:2))*16777216 + 
     +             ichar(ScratchChar2(1:1))*65536

        if (NumCols4.gt.4096) then
           print *,'*** Error in Decode_BSQ *** ',
     +                 'Number of columns too large =',NumCols4
           OK = .false.
           return
        end if

        call Transfer2 (ScratchChar2, Buff, ByteCount, InUnit)
        NumRows4 = ichar(ScratchChar2(2:2))*256+ichar(ScratchChar2(1:1))
	call Transfer2 (ScratchChar2, Buff, ByteCount, InUnit)
        NumRows4 = NumRows4 + 
     +             ichar(ScratchChar2(2:2))*16777216 + 
     +             ichar(ScratchChar2(1:1))*65536

        if (NumRows4.gt.4096) then
           print *,'*** Error in Decode_BSQ *** ',
     +                 'Number of rows too large = ',NumRows4
           OK = .false.
           return
        end if

C.......debugging statement.....
C
C        PRINT *,'DECODE_BSQ: NumCols4 = ',NumCols4,
C     +          'NumRows4 = ',NumRows4

C.......skip the rest of the first 512-byte record.....
C.......which is not needed.....

        iStart = ByteCount
        do i = iStart, 512
           call Transfer1 (ScratchChar1, Buff, ByteCount, InUnit)
        end do

C.......read in NumCols4 x NumRows4 data pixels.....
C.......convert to integer*2 and put in 'im'.....

        do i = 1,NumRows4
           do j = 1,NumCols4
              call Transfer1 (ScratchChar1, Buff, ByteCount, InUnit)
              im (j,i) = ichar (ScratchChar1)
           end do
        end do

C...normal completion of subroutine.....

        OK = .true.
        return

C...abnormal completion of subroutine.....

999	continue
        print *,'*** Error in Decode_BSQ *** ',
     +              'Unable to open file ',fin
        OK = .false.
        return
        end

C=============================================================================
	subroutine Transfer1 (OutByte, Buff, RecNum, UnitNum)
C                  ~~~~~~~~~
C This transfers a byte from Buff; if RecNum is > 512, reads a new
C Buff first.
C
C RecNum is incremented to point to the following byte after the transfer.
C=============================================================================
        character OutByte, Buff*512
	integer*4 RecNum
	integer*2 UnitNum

	if (RecNum .gt. 512) then
           read (UnitNum) Buff
           RecNum = 1
        end if

        OutByte = Buff(RecNum:RecNum)
	RecNum = RecNum + 1
        
        return
        end

C=== End of subroutine Transfer1 =============================================

C=============================================================================
	subroutine Transfer2 (Out2Byte, Buff, RecNum, UnitNum)
C                  ~~~~~~~~~
C This transfers 2 bytes from Buff; if RecNum is > 512, reads a new
C Buff first.
C
C RecNum is incremented to point to the following byte after the transfer.
C=============================================================================
        character*2 Out2Byte
        character Buff*512
	integer*4 RecNum
	integer*2 UnitNum

	if (RecNum .gt. 512) then
           read (Unitnum) Buff
           Out2Byte = Buff(1:2)
	   RecNum = 3
        else if (RecNum .eq. 512) then
           Out2Byte(1:1) = Buff(512:512)
           read (UnitNum) Buff
           Out2Byte(2:2) = Buff(1:1)
           RecNum = 2
        else
           Out2Byte = Buff(RecNum:RecNum+1)
	   RecNum = RecNum + 2
	end if

        return
        end

C=== End of subroutine Transfer2 =============================================

