C *********************************************************************
C                  UNIRAS LEGEND ROUTINES
C
C DESCRIPTION:
C    These are various routines needed by PLOTTER to create a legend,a
C    legend title and four comment lines. There are also routines for
C    plotting map titles, plotting registration marks, and some 
C    interactive I/O utility routines.
C
C AUTHOR:
C    N. Schewchenko
C    M. Tooker
C    C. Turner (I/O utilities)
C
C DATE:
C    Sept. 1, 1989
C 
C *********************************************************************


C *****************************************************************
C SUBROUTINE: Get_Legend
C
C Returns .true. if legend wanted; .false. if not.
C
C *****************************************************************

      subroutine Get_Legend(legend)

      logical legend
      character key*1

1     write(*,12)
12    format(/,' Is a LEGEND desired? (Y/N): ',$)
      read(*,'(a1)',err=1,iostat=ierr) key
      legend = (key.eq.'y'.or.key.eq.'Y')

      return
      end
      
C *****************************************************************
C    SUBROUTINE : Get_Titles
C
C    PURPOSE: This routine gets the primary and secondary map titles
C             from the user. A '$' is appended on the end of the 
C             titles as this is the delimiter character used by
C             UNIRAS.
C
C             The primary title appears underneath image in a text size
C             proportional to the size of the image.
C
C             The secondary title appears underneath the primary title
C             and is 2/3 smaller than the primary title size.        
C
C    INPUT PARAMETERS:
C             First    : true if this is the first plot
C
C    OUTPUT PARAMETERS:
C             titles - true if titles desired
C             ptitle - the primary map title
C             stitle - the secondary map title
C
C ****************************************************************
     
      subroutine Get_Titles(ptitle,stitle,titles,First)

      character ptitle*41,stitle*41,defline*41
      character key,defkey
      integer*4 msize,ssize,kline
      logical   right,titles,First,OK

      include '[bcvax.fort]txtfnct.cns'

C.................................................................

      print *,' '

      if ((First).or.(.not.Titles)) then
         defkey = 'N'
      else
         defkey = 'Y'
      end if

      write (*,10) defkey
10    format (' Do you want titles plotted at the '
     +        'bottom center of the image? (Y/N) [',a1,']: ',$)
      read (*,'(a1)') key

      if (lchar(key).eq.0) then
         key = defkey
      end if

      if ((key.ne.'y').and.(key.ne.'Y')) then
         titles = .false.
         return
      end if

      titles = .true.

      print *,' '
      print *,'-------------------------------------------------------'
      print *,'        The PRIMARY map title will be centered '
      print *,'                underneath the map.'
      print *,' The SECONDARY map tile will be centered underneath'
      print *,'      the primary title in smaller sized text.'
      print *,' '
      print *,'     Each title can be up to 40 characters long.'
      print *,'-------------------------------------------------------'
      print *,' '
      print *,' '
      kline = 0
      right = .FALSE.

        do while (.not.right)

           if (kline.ne.2) then

C............default is old title, minus '$' at end.....

33            defline = '                                         '
              if (.not.First) then 
                 defline = ptitle(1:(lchar(ptitle) - 1)) // defline
              end if

              write (*,1) defline
1     	      format(/,' PRIMARY TITLE (Hit <Enter> for default, ',
     +                 '$ <Enter> for blank line)',/,
     +                 ' ["', a<lchar(defline)>, '"]: ',$)
      	      read(*,'(a40)',err=33,iostat=ierr) ptitle

              if (lchar(ptitle).eq.0) then
                 ptitle = defline
              else if (ptitle(1:1).eq.'$') then
                 ptitle = '                                         '
              end if

              call Make_String(41,ptitle,msize)

           endif

           if (kline.ne.1) then

77            defline = '                                         '
              if (.not.First) then 
                 defline = stitle(1:(lchar(stitle) - 1)) // defline
              end if

              write (*,2) defline
2     	      format(/,' SECONDARY TITLE (Hit <Enter> for default, ',
     +                 '$ <Enter> for blank line)',/,
     +                 ' ["', a<lchar(defline)>, '"]: ',$)
      	      read(*,'(a40)',err=33,iostat=ierr) stitle

              if (lchar(stitle).eq.0) then
                 stitle = defline
              else if (stitle(1:1).eq.'$') then
                 stitle = '                                        '
              end if

              call Make_String(41,stitle,ssize)

           endif
 
           write (*,*) SBLACK,CLR,' '
           write (*,*) DBLW,UL,'Map Titles',RST
           print *,' '

220        continue

           print *,' '
           print *,'(1) Primary title  : "',ptitle(1:lchar(ptitle)-1),
     +              '"'
           print *,'(2) Secondary title: "',stitle(1:lchar(stitle)-1),
     +              '"'

           print *,' '
           call ENTINT (' Enter TITLE (1 or 2) to change
     + (Enter 0 if okay) ',.true.,0,kline,OK)

           if (((kline.lt.0).or.(kline.gt.2)).or.(.not.OK)) then
               print *,'**ERROR** invalid response'
               go to 220
           endif
           right = (kline.eq.0)
        end do        

      return 
      end 

C *****************************************************************
C    SUBROUTINE : Write_Titles
C
C    PURPOSE: This routine writes the primary and secondary map titles.
C             The primary title appears underneath image in a text size
C             proportional to the size of the image.
C
C             The secondary title appears underneath the primary title
C
C    INPUT PARAMETERS:
C             ptitle - the primary map title
C             stitle - the secondary map title
C             plpix  - output image grid size
C             xsize,
C             ysize  - size of plot in mm
C             xorig  - the origin in the x direction in mm
C             yorig  - the origin in the y direction in mm
C             xtitle - total height of titles
C             plotter - device on which plot is output
C
C ****************************************************************
 
      subroutine Write_Titles(ptitle,stitle,plpix,xsize,ysize,
     +                        xorig,yorig,xtitle,plotter)

      character ptitle*41,stitle*41
      real*4 plpix,xorig,yorig,length,mtexth,stexth
      real*4 xsize,ysize,xtitle
      integer plotter

      call gcharf('SIMP')		 	! set the font to SIMP

C.....primary title........................................................

      mtexth = xsize/20.                        ! get height
      call gchar(ptitle,length,8888.,mtexth)    ! get the length of the string

      do while (length.gt.ysize)                ! reduce height if too long
         mtexth = mtexth * 0.9
         call gchar(ptitle,length,8888.,mtexth)
      end do

      x = xorig + xsize + (1.5 * mtexth)        ! set up origin
      y = yorig + ((ysize - length) / 2.)

      call gchara (90)                          ! rotate text 90 degrees

      call gchar(ptitle,x,y,mtexth)		! write primary title

C.....secondary title........................................................
   
      stexth = mtexth * 0.66667                 ! 2nd title 2/3 size of 1st
      call gchar(stitle,length,8888.,stexth)	! get length of the string

      do while (length.gt.ysize)                ! reduce height if too long
         stexth = stexth * 0.9
         call gchar(stitle,length,8888.,stexth)
      end do

      x = x + stexth + (stexth / 0.75)          ! set up origin
      y = yorig + ((ysize - length) / 2.)

      call gchara(90)			        ! rotate text 90 degrees

      call gchar(stitle,x,y,stexth)		! write secondary title

      xtitle = x - (xorig + xsize)              ! calculate total height

      return
      end

C **********************************************************************
C SUBROUTINE: Get_Comments
C
C PURPOSE:    This routine gets four comment lines from the user to
C             be plotted underneath the legend. 
C
C INPUT PARAMETERS:
C             First    : true if this is the first plot
C
C OUTPUT PARAMETERS:
C             Line     : 4 comment line strings
C             wants    : true if lines to be plotted
C
C *********************************************************************

      SUBROUTINE Get_Comments (line,Wants,First)

      character*36 line(4),defline
      character key,defkey
      logical right,wants,First,OK
      integer*4 isize,startline,endline

      include '[bcvax.fort]txtfnct.cns'

C.................................................................

      right = .FALSE.

C**********************************************************************
C            Determine if user wants comments
C**********************************************************************

      print *,' '

      if ((First).or.(.not.Wants)) then
         defkey = 'N'
      else
         defkey = 'Y'
      end if

      write (*,10) defkey
10    format (' Do you want comment lines plotted? (Y/N) [',
     +          a1,']: ',$)
      read (*,'(a1)') key

      if (lchar(key).eq.0) then
         key = defkey
      end if

      if ((key.ne.'y').and.(key.ne.'Y')) then
         wants = .false.
         return
      end if

      wants = .true.

C**********************************************************************
C            Get comment lines from user
C**********************************************************************

      print *,' '
      print *,'-------------------------------------------------------'
      print *,' Four COMMENT LINES can be placed to the lower right,'
      print *,'                    of the image.'
      print *,'      Each line can be up to 35 characters long.'
      print *,'-------------------------------------------------------'
      print *,' '

      startline = 1
      endline = 4
 
      do while (.not. right)

         do i = startline,endline

33          defline = '                                    '
            if (.not.First) then
               defline = line(i)(1:lchar(line(i)) - 1) // defline
            end if

            write(*,1) i, defline 
1           format(' Line ',i1,' (Hit <Enter> for default, ',
     +              '$ <Enter> for blank line)',/,
     +              ' ["', a<lchar(defline)>, '"]: ',$)
            read(*,'(a35)',err=33) line(i)

            if (lchar(line(i)).eq.0) then
               line(i) = defline
            else if (line(i)(1:1).eq.'$') then
               line(i) = '                                    '
            end if

            call Make_String (36,line(i),isize)

         enddo

C**********************************************************************
C            Check if user likes the lines entered
C**********************************************************************

         write(*,*) sblack,clr,' '
         write(*,*) dblw,ul,'Comment Lines',rst
         print *,' '
         print *,' '

20       continue

         do i = 1,4
           print 5,i,line(i)
5          format(' line ',i1,': "',a<lchar(line(i))-1>,'"')
         enddo

         print *,' ' 
         call ENTINT (' Enter LINE # to change (Enter 0 if okay) ',
     +                .true.,0,startline,OK)

         if (((startline.lt.0).or.(startline.gt.4)).or.(.not.OK)) then
            print *,'**ERROR** Invalid Choice!'
            go to 20
         endif

         right = (startline .eq. 0)
         endline = startline

      end do

      return
      end

C **********************************************************************
C SUBROUTINE: Write_Comments
C PURPOSE:    Plots 4 comment lines. The comments are scaled to the 
C             size of the image and to fit in the bottom 1/5 of the image.
C
C INPUT PARAMETERS:
C             Line     : 4 strings of comments
C             xorig    : the origin of image in x direction
C             yorig    : the origin of image in y direction  
C             xsize    : Size in mm of image in x direction
C             ysize    : Size in mm of image in y direction 
C             plotter  : device on which plot being made
C
C OUTPUT PARAMETERS:
C             C_Length : length in mm of longest comment
C
C *********************************************************************

      subroutine Write_Comments (line,xsize,ysize,xorig,yorig,
     +                           c_length,plotter)

      character*36 line(4)
      real*4 xsize,ysize,xorig,yorig,c_length
      integer plotter
      real height,length

C**********************************************************************
C            Prepare to plot comments
C**********************************************************************

      call gcharf('SIMP')

C comment text size = 1/2 of legend title size

      height = amin1 ((xsize/40.0),4.0)

C position x and y to start writing comment lines

      x = xorig + (0.8 * xsize) + height
      y = yorig + ysize + 3.0

      c_length = 0.0
      length = 0.0

C************************************************************************
C          Plot the 4 comments
C************************************************************************

      do i = 1,4

C........determine length of comment in mm.....

         call gchar(line(i),length,8888.,height)
         if (length .gt. c_length) then
            c_length = length
         endif

C........rotate text.....

         call gchara (90)

C........plot the characters.....

         call gchar(line(i),x,y,height)

C........adjust origin for next line.....

         x = x + (1.5 * height)

      enddo

      c_length = c_length + 3.0   ! add 3mm space to longest comment length

      return
      end

C **********************************************************************
C SUBROUTINE: Make_String
C PURPOSE:    This routine take the character string 'string' and
C             adds a '$' (the text delimiter used by RASPAK) at the
C             end of the string.
C PARAMETERS:
C             str_size : The declared size of the string
C             string   : a character array
C             size     : the size of the string upto the '$' character
C
C **********************************************************************

      SUBROUTINE Make_String(str_size,string,NumChars)

      integer*4 str_size,NumChars
      character*(*) string

C.....strip leading blanks.....

      j = 1
      do while ((string(j:j).eq.' ').and.(j.le.str_size))
         j = j + 1
      end do

      string = string(j:str_size) // 
     +          '                                  '

C.....place a '$' after the last non-blank character.....

      j = str_size - 1
      do while ((string(j:j).eq.' ').and.(j.gt.0))
         j = j - 1
      end do

      string(j+1:j+1) = '$'

C.....send back the number of non-blank chars.....

      NumChars = j

      return
      end

C **********************************************************************
C SUBROUTINE: Plot_Registration
C PURPOSE:    This routine plots registration marks in the four corners
C             of the image.
C PARAMETERS:
C             xorig, yorig         - top left corner of image in mm
C             largest_x, largest_y - offset of lower right corner in mm
C                                    These are updated and returned.
C             plotter              - type of output device
C             plpix                - size of plotted pixel in mm
C             Is_RGB               - true if RGB; false if CMY
C
C *********************************************************************

      SUBROUTINE Plot_Registration (xorig,yorig,largest_x,largest_y,
     +                              plpix,plotter,Is_RGB)

      real*4 xorig,yorig,largest_x,largest_y,plpix
      integer plotter
      real*4 x,y,offset
      integer Black, NearBlack
      real HalfLength

      logical Is_RGB

C***********************************************************************
C                   set up reg. mark constants
C***********************************************************************

      NearBlack = 102       ! color table indices of near black
      Black = 103           !    & black
      HalfLength = 5.0      ! = 1/2 height/width of reg. mark in mm
      Thickness = 0.5       ! thickness of each line in mm
      offset = 5.0          ! offset in mm from border at at which to plot

C***********************************************************************
C                     change color table
C***********************************************************************

      if (Is_RGB) then

         call gcmode (-1,iamode)
         call rcmode ('RGB',255)

C........for Teltronix screen plots, simulate with blue & red.....

         if (plotter.eq.4) then
            call gcolor (NearBlack,0,255,255,1)      ! load pure cyan color
            call gcolor (Black,255,0,0,1)            ! load orange-red color

C........otherwise, make black & near black colors   .....
C........to come out on cyan, magenta, & yellow plots.....

         else
            call gcolor (NearBlack,0,50,100,1)       ! load 'almost black' color
            call gcolor (Black,0,0,0,1)              ! load 'pure black' color
         end if


      else

         call gcmode (-2,iamode)
         call rcmode ('CMY',255)

C........for Teltronix screen plots, simulate with blue & red.....

         if ((plotter.eq.4).or.(plotter.eq.5)) then
            call gcolor (NearBlack,255,0,0,1)      ! load pure cyan color
            call gcolor (Black,0,255,255,1)        ! load orange-red color

C........otherwise, make black & near black colors   .....
C........to come out on cyan, magenta, & yellow plots.....

         else
            call gcolor (NearBlack,150,200,255,1)    ! load 'almost black' color
            call gcolor (Black,255,255,255,1)        ! load 'pure black' color
         end if

      end if

C***********************************************************************
C plot 4 marks
C***********************************************************************

C.....upper left corner.....

      x = xorig - offset
      y = yorig - offset
      call Make_Reg_Mark (x,y,plpix,NearBlack,Black,HalfLength,
     +                    Thickness)

C.....lower left corner

      x = largest_x + offset
      y = yorig - offset
      call Make_Reg_Mark (x,y,plpix,NearBlack,Black,HalfLength,
     +                    Thickness)

C.....lower right corner.....

      x = largest_x + offset
      y = largest_y + offset
      call Make_Reg_Mark (x,y,plpix,NearBlack,Black,HalfLength,
     +                    Thickness)

C.....upper right corner.....

      x = xorig - offset
      y = largest_y + offset
      call Make_Reg_Mark (x,y,plpix,NearBlack,Black,HalfLength,
     +                    Thickness)

C.....update largest coord. values to show reg. marks.....

      largest_x = largest_x + offset + HalfLength
      largest_y = largest_y + offset + HalfLength

      return
      end

C **********************************************************************
C SUBROUTINE: Make_Reg_mark
C
C PURPOSE:    This routine plots a cross-shaped registration mark
C             in near-black and black at the given coordinates.
C
C PARAMETERS:
C             x,y        - center coordinate for reg. mark
C             Black, 
C             NearBlack  - color table entries used
C             plpix      - size of single output pixel in mm
C             HalfLength - 1/2 length of each line in cross
C             Thickness  - thickness in mm of each line to be drawn
C
C *********************************************************************

      SUBROUTINE Make_Reg_Mark (x,y,plpix,NearBlack,Black,HalfLength,
     +                          Thickness)

      real*4 x,y,plpix
      integer NearBlack,Black
      real xArr(2),yArr(2),Thickness, HalfLength

C.....set line thickness & color for center cross.....

      call gwicol (Thickness,NearBlack)  ! line is solid by default

C.....draw vertical center stroke.....

      xArr(1) = x
      xArr(2) = x
      yArr(1) = y - HalfLength
      yArr(2) = y + HalfLength

      call gvect (xArr,yArr,2)

C.....draw horiz. center stroke.....

      xArr(1) = x - HalfLength
      xArr(2) = x + HalfLength
      yArr(1) = y
      yArr(2) = y

      call gvect (xArr,yArr,2)

C.....set up outer cross line thickness & color

      call gwicol (Thickness,Black)

C.....draw upper left "L".....

      xArr(1) = x - Thickness
      yArr(1) = y - Thickness
      xArr(2) = x - HalfLength
      yArr(2) = yArr(1)

      call gvect (xArr,yArr,2)

      xArr(2) = xArr(1)
      yArr(2) = y - HalfLength

      call gvect (xArr,yArr,2)

C.....draw upper right "L".....

      xArr(1) = x + Thickness
C     (yArr(1) same)      
      xArr(2) = x + HalfLength
      yArr(2) = yArr(1)

      call gvect (xArr,yArr,2)

      xArr(2) = xArr(1)
      yArr(2) = y - HalfLength

      call gvect (xArr,yArr,2)

C.....draw lower right "L".....

C     (xArr(1) same)
      yArr(1) = y + Thickness
      xArr(2) = x + HalfLength
      yArr(2) = yArr(1)

      call gvect (xArr,yArr,2)

      xArr(2) = xArr(1)
      yArr(2) = y + HalfLength

      call gvect (xArr,yArr,2)

C.....draw lower left "L".....

      xArr(1) = x - Thickness
C     (yArr(1) same)
      xArr(2) = x - HalfLength
      yArr(2) = yArr(1)

      call gvect (xArr,yArr,2)

      xArr(2) = xArr(1)
      yArr(2) = y + HalfLength

      call gvect (xArr,yArr,2)

      return
      end

C **********************************************************************
C SUBROUTINE: Plot_legend2
C
C PURPOSE:    Plots a legend and scale on the right of the image.
C
C INPUT PARAMETERS:
C             Title       - string containing 'Legend' or other title
C             NumThemes   - number of themes in legend
C             Theme       - array of titles for each theme
C             im_width    - width of real area in meters
C             orient      - heading of true north in degrees
C             CTable      - color table information
C             xorig,yorig - origin of plot in mm
C             xsize,ysize - size of plot in mm
C             Is_RGB      - true if RGB; false if CMY
C             plotter     - output device type
C             leg_length  - length in mm of longest part of legend
C
C *********************************************************************

      SUBROUTINE Plot_Legend2 (Title,NumThemes,Theme,im_width,orient,
     +                         ctable,xorig,yorig,xsize,ysize,
     +                         Is_RGB,plotter,leg_length)

      character*25 Theme(12)
      character Title*31
      integer   NumThemes,plotter
      integer   ctable(12,3)
      real*4 xorig,yorig,xsize,ysize,leg_length
      real*4 im_width,scale,orient
      real*4 big_y
      logical Is_RGB
      character key*1,coded_scale*9,scale_str*15
      real xArr(2),yArr(2)

      real rect_h,rect_w,edge_y,
     +     title_x,title_y,title_h,
     +     theme_x,theme_y,theme_h,
     +     scale_x,scale_y,scale_h,
     +     orient_x,orient_y,orient_h,
     +     delta_x,delta_y,
     +     length,line_x,legsize

C.....initialize data.....

      legsize = amin1 ((xsize/2),150.0)   ! entire legend no more than 15 cm high
      
      edge_y = yorig + ysize + 3.0        ! main y-offset for legend

      title_h = 0.8 * (legsize/15)
      title_x = xorig + (1.3 * title_h)   ! title x-offset, height

      line_x = xorig + 1.6 * (legSize/15) ! x-offset of line under title

      rect_h = legsize/15                 ! rectangle heigth, width
      rect_w = 1.5 * rect_h

      theme_y = edge_y + rect_w + 1.5     ! theme titles' y-offset, height
      theme_h = 0.03 * legsize

      orient_h = AMIN1((0.025*xsize),5.0) ! size of 'N'
      orient_x = xorig + (0.5 * xsize)    ! base position of orientation

      scale_x = xorig + (0.71667 * xsize) ! scale x-offset, height
      scale_h = AMIN1((0.05 * legsize),5.0)

C.....set up black color for drawing lines, etc.....

      if (Is_RGB) then
         call gcmode (-1,iamode)
         call rcmode ('RGB',255)                  ! set color mode
         if ((plotter.eq.4).or.(plotter.eq.5)) then
            call gcolor (104,255,255,255,1)
         else
            call gcolor (104,0,0,0,1)                  ! load black
         end if
      else
         call gcmode (-2,iamode)
         call rcmode ('CMY',255)                  ! set color mode
         if ((plotter.eq.4).or.(plotter.eq.5)) then
            call gcolor (104,0,0,0,1)                  ! load black
         else
            call gcolor (104,255,255,255,1)
         end if
      end if

C.....load theme colors starting at entry.....
c.....103 in UNIRAS color table.....

      do i = 1,NumThemes
         call gcolor (i+104,ctable(i,1),ctable(i,2),ctable(i,3),1)
      end do 

C.....plot legend title.....

      call gcharf('SIMP')

      call gchara (90)
      call gchar(title,title_x,edge_y,title_h)
      call gchar(title,length,8888.,title_h)   ! get the length of the string

      big_y = edge_y + length                  ! largest y-value taken so far

C.....plot theme boxes and titles.....

      rect_x = xorig + (0.2 * legsize)         ! first box
      do i = 1,NumThemes
         call rrect (rect_x,edge_y,rect_x+rect_h,edge_y+rect_w,
     +               i+104,0)                             ! plot box
         theme_x = rect_x + (1.3*theme_h)                 ! position text
         call gchara (90)                                 ! rotate text 90 deg.
         call gchar (theme(i),theme_x,theme_y,theme_h)    ! plot title
         call gchar (theme(i),length,8888.,theme_h)       ! get the length
         big_y = AMAX1 (big_y,(theme_y+length))           ! save if longest
         rect_x = rect_x + rect_h                         ! move to next box
      end do

C.....plot orientation arrow.....

      if (orient.ge.0.0) then

         if ((orient.lt.45.).or.(orient.ge.315.)) then    ! point from bottom up
            if (orient.lt.45) then
               xArr(1) = orient_x + (0.15 * xsize)
               yArr(1) = edge_y + 10.0
            else
               xArr(1) = orient_x + (0.15 * xsize)
               yArr(1) = edge_y + (0.15 * xsize)
            end if
            delta_x = - 0.1 * xsize
            delta_y = - (TAND(orient) * delta_x)
            orient_x = xArr(1) + delta_x - orient_h
            orient_y = yArr(1) + delta_y
         else if ((orient.ge.45.).and.(orient.lt.135.)) then ! point l. to r.
            xArr(1) = orient_x + (0.1 * xsize)
            yArr(1) = edge_y
            delta_y = AMIN1((0.08*xsize),80.0)
            if ((orient.lt.89.999).or.(orient.gt.90.001)) then
               delta_x = (-1 / TAND(orient)) * delta_y
            else
               delta_x = 0
            end if
            orient_x = xArr(1) + delta_x
            orient_y = yArr(1) + delta_y + orient_h
         else if ((orient.ge.135.).and.(orient.lt.225.)) then  ! point top down
            if (orient.lt.180) then
               xArr(1) = orient_x + (0.05 * xsize)
               yArr(1) = edge_y + 10.0
            else
               xArr(1) = orient_x + (0.05 * xsize)
               yArr(1) = edge_y + (0.15 * xsize)
            end if
            delta_x = 0.1 * xsize
            delta_y = - (TAND(orient) * delta_x)
            orient_x = xArr(1) + delta_x + (1.5 * orient_h)
            orient_y = yArr(1) + delta_y
         else ! ((orient.ge.225.).and.(orient.lt.315.))   ! point from r. to l.
            xArr(1) = orient_x + (0.1 * xsize)
            yArr(1) = edge_y + AMIN1((0.2*xsize),95.0)
            delta_y = - ( AMIN1((0.12*xsize),80.0) )
            if ((orient.lt.269.999).or.(orient.gt.270.001)) then
               delta_x = (-1 / TAND(orient)) * delta_y
            else
               delta_x = 0
            end if
            orient_x = xArr(1) + delta_x
            orient_y = yArr(1) + delta_y - orient_h
         end if

         xArr(2) = xArr(1) + delta_x
         yArr(2) = yArr(1) + delta_y

         call gwicol (0.5,104)          ! draw the arrow line
         call gvect (xArr,yArr,2)

         len = SQRT ( (delta_x * delta_x) + 
     +                (delta_y * delta_y) )

C                                       ! make arrow head with 3 dots at end

         xArr(1) = xArr(1) + 0.97 * delta_x  ! big dot
         yArr(1) = yArr(1) + 0.97 * delta_y
         call gwicol (0.04*len,104)
         call gdot (xArr,yArr,1)
         xArr(1) = xArr(1) + 0.02 * delta_x  ! medium dot
         yArr(1) = yArr(1) + 0.02 * delta_y
         call gwicol (0.02*len,104)
         call gdot (xArr,yArr,1)
         xArr(1) = xArr(1) + 0.01 * delta_x  ! little dot
         yArr(1) = yArr(1) + 0.01 * delta_y
         call gwicol (0.01*len,104)
         call gdot (xArr,yArr,1)

         if ((orient_x-orient_h).lt.(xorig+legsize)) then
            orient_h = 0.8 * orient_h                   ! height of 'N'
         end if

         call gchara (90)
         call gchar ('N$',orient_x,orient_y,orient_h)   ! plot 'N' for north

      end if

C.....plot scale.....

      if (im_width.gt.0.0) then
         
         length = 0.33333 * ysize

         xArr(1) = scale_x
         yArr(1) = edge_y
         xArr(2) = xArr(1)
         yArr(2) = edge_y + length

         call gwicol (0.5,104)
         call gvect (xArr,yArr,2)

         xArr(1) = xorig + (0.7 * xsize)
         yArr(1) = edge_y
         xArr(2) = xorig + (0.7333 * xsize)
         yArr(2) = edge_y

         call gvect (xArr,yArr,2)

         xArr(1) = xorig + (0.7 * xsize)
         yArr(1) = edge_y + length
         xArr(2) = xorig + (0.7333 * xsize)
         yArr(2) = yArr(1)

         call gvect (xArr,yArr,2)

         scale = im_width / 3
         if (scale.lt.1000.) then
            encode (7,75,coded_scale) scale
75          format (f5.1)
            scale_str = coded_scale( fchar(coded_str) 
     +                               : lchar(coded_str) ) // ' m$'
         else
            encode (7,75,coded_scale) scale/1000
            scale_str = coded_scale( fchar(coded_str)
     +                               : lchar(coded_str) ) // ' km$'
         end if

         call gchar (scale_str,length,8888.,scale_h)      ! get the length
         scale_y = (edge_y + ((0.3333 * ysize) / 2)) - (length / 2)

         scale_x = scale_x + (1.8 * scale_h)              ! plot below line

         call gchara (90)                                 ! rotate text 90 deg.
         call gchar (scale_str,scale_x,scale_y,scale_h)   ! plot scale string

         big_y = AMAX1 (big_y,(edge_y+length),
     +                  (edge_y+(0.3333 * ysize)))

      end if

C.....underline title to max. length.....

      call gwicol (0.5,104)
      xArr(1) = line_x
      xArr(2) = xArr(1)
      yArr(1) = edge_y
      yArr(2) = big_y

      call gvect (xArr,yArr,2)

C.....calculate furthest extension of legend.....

      leg_length = (big_y - edge_y) + 3.0

      return
      end

C **********************************************************************
C SUBROUTINE: Get_legend2
C
C PURPOSE:    This routine gets information from user to create a
C             legend and scale on the right of the plotted image.
C
C INPUT PARAMETERS:
C             Is_RGB - true if RGB; false if CMY
C             data_type - 1 if channels; 2 if color theme entries
C             ysize - size of plot in mm; used to calculate cm_scale
C             First - true if this is the first plot
C
C OUTPUT PARAMETERS:
C             wants - true if legend wanted
C             Title - string containing 'Legend' or other title
C             NumThemes - number of themes in legend
C             Theme - array of titles for each theme
C             scale - width of image in meters
C             orient - orientation of north arrow in degrees
C             CTable - color table information
C
C *********************************************************************

      SUBROUTINE Get_Legend2 (wants,Title,NumThemes,Theme,scale,orient,
     +                        ctable,Is_RGB,data_type,ysize,First)

      logical      wants,Is_RGB,Checking,First
      character*25 Theme(12),deftheme
      character*31 Title*31,deftitle*31
      integer      NumThemes
      integer      data_type
      integer      ctable(12,3)
      real*4       width,ysize,scale,orient
      integer*4    TitleSize,ThemeSize(12)
      character    key,defkey

      logical      OK

      include '[bcvax.fort]txtfnct.cns'

C.................................................................

C.....find out if legend wanted.....

5     if ((First).or.(.not.Wants)) then
         defkey = 'N'
      else
         defkey = 'Y'
      end if

      write (*,10) defkey
10    format (/,' Do you want a legend? (Y/N) [',a1,']: ',$)
      read (*,'(a1)',err=5) key

      i = lchar(key)

      if (lchar(key).eq.0) then
         key = defkey
      end if

C.....no legend...quit.....

      if ((key.ne.'y').and.(key.ne.'Y')) then
         if (First) title = 'Legend$'      ! default title for poss. later plot
         wants = .false.
         return
      end if

C.....get legend information from user.....

      wants = .true.
      Checking = .false.

C.....get legend title.....

11    deftitle = '                               '
      if ((First).or.(title(1:lchar(title)).eq.'Legend$')) then
         deftitle = 'Legend' // deftitle
      else
         deftitle = title(1:lchar(title) - 1) // deftitle
      end if

13    write (*,14) deftitle
14    format (/,' TITLE (Hit <Enter> for default, ',
     +          '$ <Enter> for blank line)',/,
     +          ' ["', a<lchar(deftitle)>, '"]: ',$)
      read (*,'(a30)',err=13) Title

      if (lchar(title).eq.0) then
         title = deftitle
      else if (title(1:1).eq.'$') then
         title = '                              '
      end if

      call Make_String (31,Title,TitleSize)

      if (Checking) goto 80

C.....get # of themes in legend.....

15    if ((First).and.(.not.Checking)) NumThemes = 0

      write (*,20)
20    format (/,' How many themes do you want in the legend? ',$)
      call ENTINT ('(0-12): ',.true.,NumThemes,NumThemes,OK)

      if (((NumThemes.lt.0).or.(NumThemes.gt.12)).or.(.not.OK)) then
         print *,'*** INVALID *** You must enter 0-12'
         goto 15
      end if

C.....get theme color values.....

      do i = 1,NumThemes
         print *,' '
         print *,'THEME ',i,':'

23       continue

25       write (*,30)
30       format (' Enter theme color values (0-255)')

         if (Is_RGB) then
            call ENTINT ('Red   ',.true.,ctable(i,1),ctable(i,1),OK)
               if (.not.OK) goto 32
            call ENTINT ('Green ',.true.,ctable(i,2),ctable(i,2),OK)
               if (.not.OK) goto 32
            call ENTINT ('Blue  ',.true.,ctable(i,3),ctable(i,3),OK)
               if (.not.OK) goto 32
         else
            call ENTINT ('Cyan    ',.true.,ctable(i,j),ctable(i,j),OK)
               if (.not.OK) goto 32
            call ENTINT ('Magenta ',.true.,ctable(i,j),ctable(i,j),OK)
               if (.not.OK) goto 32
            call ENTINT ('Yellow  ',.true.,ctable(i,j),ctable(i,j),OK)
               if (.not.OK) goto 32
         end if
         goto 33
32          print *,'*** ERROR *** Values must be between 0 ',
     +              ' and 255.'
            goto 23
  
33       continue
         if ((((ctable(i,1).lt.0).or.(ctable(i,1).gt.255)).or.
     +      ((ctable(i,2).lt.0).or.(ctable(i,2).gt.255))).or.
     +      ((ctable(i,3).lt.0).or.(ctable(i,3).gt.255)))
     +   then
            print *,'*** ERROR *** Values must be between 0 ',
     +              ' and 255.'
            goto 23
         end if

C........get theme title.....

45       defTitle = '                        '
         if ((.not.First).or.(Checking)) then
            deftitle = Theme(i)(1:lchar(theme(i)) - 1) // deftitle
         end if            

         write (*,50) deftitle
50       format (' THEME TITLE (Hit <Enter> for default, ',
     +             '$ <Enter> for blank line)',/,
     +             ' ["', a<lchar(deftitle)>, '"]: ',$)
         read (*,'(a24)',err=45) Theme(i)

         if (lchar(Theme(i)).eq.0) then
            Theme(i) = deftitle
         else if (Theme(i)(1:1).eq.'$') then
            Theme(i) = '                              '
         end if

         call Make_String (25,Theme(i),ThemeSize(i))

      end do

      if (Checking) goto 80

C.....get scale.....

55    continue

      if (((First).and.(.not.Checking)).or.(scale.eq.0.0)) then
         defkey = 'N'
      else
         defkey = 'Y'
      end if

      write (*,60) defkey
60    format (//,' Do you want a scale included in the legend?',
     +           ' (Y/N) [',a1,']: ',$)
      read (*,'(a1)',err=55) key

      if (lchar(key).eq.0) then
         key = defkey
      end if
         
      if ((key.eq.'y').or.(key.eq.'Y')) then
65       continue
         call ENTREAL (' Width of image in meters ',.true.,
     +                  scale,scale,OK)
         if ((scale.le.0.0).or.(.not.OK)) then
            print *,'** ERROR *** Image cannot have zero or ',
     +              'negative width.'
            goto 65
         end if

      end if

      if (Checking) goto 80

C.....get orientation.....

73    continue

      if (((first).and.(.not.Checking)).or.(orient.eq.-1.0)) then
         defkey = 'N'
      else
         defkey = 'Y'
      end if

      write (*,75) defkey
75    format (//,' Do you want a north arrow included ',
     +           'in the legend? (Y/N) [',a1,']: ',$)
      read (*,'(a1)',err=55) key

      if (lchar(key).eq.0) then
         key = defkey
      end if

      if ((key.eq.'y').or.(key.eq.'Y')) then

76       if ((orient.lt.0.0).or.(orient.gt.360.0)) then
            orient = 0.0
         end if

         write (*,77)
77       format (' Enter the bearing of north on your map in degrees ',/,
     +           ' (eg. straight up is 0 or 360, right is 90')
         call ENTREAL (' down is 180, left is 270) ',
     +                              .true.,orient,orient,OK)

         if (((orient.lt.0.0).or.(orient.gt.360.0)).or.(.not.OK)) then
            print *,'** ERROR *** Must be between 0 and 360'
            goto 76
         end if

      else
         orient = -1.0
      end if

C.....check if user likes the information that has been entered.....

80    continue

C.....show what the user has entered for legend.....

      write (*,*) SBLACK,CLR,' '
      write (*,*) DBLW,UL,'Map Legend',RST
      print *,' '
      write (*,81) Title(1:TitleSize)
81    format (' (1) Legend Title    : "',a<TitleSize>,'"')
      print *,' '
      print *,'(2) Legend Themes'
      print *,'    Number of Themes: ',NumThemes

      do i = 1,NumThemes
         write (*,82) i,ctable(i,1),ctable(i,2),ctable(i,3),
     +                Theme(i)(1:ThemeSize(i))
82       format ('     Theme ',i2,' Color: (',i3,' ',i3' ',i3,')',
     +           ' Title: "',a<ThemeSize(i)>,'"')
      end do

      print *,' '
      if (scale.gt.0.0) then
         print *,'(3) Scale included; Map width in m = ',scale
      else
         print *,'(3) Scale not included'
      end if

      print *,' '
      if (orient.ge.0.0) then
         print *,'(4) Orientation arrow included; ',
     +           'North bearing in degrees = ',orient
      else
         print *,'(4) Orientation arrow not included'
      end if

C.....ask what user wants to change.....

      print *,' '
      print *,'Which of these would you like to change?'
      call ENTINT (' Enter choice 1-4, or 0 to continue: ',.true.,
     +               0,iChoice,OK)

      if (((iChoice.gt.4).or.(iChoice.lt.0)).or.(.not.OK)) then
         goto 80
      else  
         Checking = .true.
         if (iChoice.eq.1) then
            goto 11
         else if (iChoice.eq.2) then
            goto 15
         else if (iChoice.eq.3) then
            goto 55
         else if (iChoice.eq.4) then
            goto 73
         end if
      end if

      return
      end

C *****************************************************************
C    SUBROUTINE : Register_Query
C
C    PURPOSE: Finds out from the user if registration marks are
C             wanted. 
C
C    INPUT PARAMETERS:
C             first - true if this is the first plot
C
C    OUTPUT PARAMETERS:
C             register - true if wanted
C
C ****************************************************************

      subroutine Register_Query (Register,First)

      logical Register,First
      character key,defkey

C..............................................................................

100   if ((First).or.(.not.Register)) then
         defkey = 'N'
      else
         defkey = 'Y'
      end if

      write (*,102) defkey
102   format(/,' Do want registration marks plotted? (Y/N) ',
     +         '[',a1,']: ',$)
      read(*,'(a1)',err=100) key

      if (lchar(key).eq.0) then
         key = defkey
      end if

      Register = ((key.eq.'y').or.(key.eq.'Y'))

      return
      end

c *******************************************************************
c VAX UTITITY functions and procedures
c ------------------------------------
c (Chris Turner, May-August 1988)
c
c *******************************************************************

c *********************************************************************
c ENTINT: A subroutine which gets an integer FINAL_INTEGER from the
c ------  user.
c         A default integer may be specifed by setting DEFAULT_ON to
c         true and setting DEFAULT_INTEGER to the appropriate value.
c         A prompt may be specified by setting the string DESCR
c         accordingly.
c *********************************************************************

      subroutine ENTINT(descr,default_on,default_integer,
     +                  final_integer,OK)

c Constants

      include 'chardef.cns'

c Parameters

      character*(*) descr
      logical       default_on,OK
      integer*4     default_integer
      integer*4     final_integer

c Work variables

      character*79  outstr
      character*32  intstr,tempstr
      integer*4     first_digit_pos,last_digit_pos

c External functions

      integer*4     FCHAR,LCHAR
      real*4        CHR_RL

c Create the prompt with default if desired

      outstr = descr
      if (default_on) then
         tempstr = BLANK
         write (tempstr,*) default_integer
         first_digit_pos = FCHAR(tempstr)
         last_digit_pos  = LCHAR(tempstr)
         outstr = outstr(:LCHAR(outstr)) // ' [ ' //
     +            tempstr(first_digit_pos:last_digit_pos) // ' ]'
      endif
      outstr = outstr(:LCHAR(outstr)) // ': '

c Display the prompt and allow user to enter his/her choice
c [RETURN] = accept default

      write (*,1000) outstr(:LCHAR(outstr))
1000  format(1x,a,$)

      read (*,'(a)') intstr

c Process response

      if (intstr .eq. BLANK .and. default_on) then
         final_integer = default_integer
         OK = .true.
         return
      endif

      decode (lchar(intstr),10,intstr(1:lchar(intstr)),
     +                                 err=20) final_integer
10    format (i<lchar(intstr)>)
      OK = .true.
      return

20    final_integer = default_integer
      OK = .false.
      return

      END

c *******************************************************************
c CHR_RL: character to real
c -------
c A function which converts a real or integer number contained
c in the string CHRNUM to a real number.
c The corresponding real number is returned as CHR_RL.
c *******************************************************************

      real*4 function CHR_RL(chrnum)

c Constants

      include 'chardef.cns'

c Parameters

      character*(*) chrnum

c Work variables

      character*1   curchr
      integer*4     fpos,lpos,decpos
      integer*4     ascii
      integer*4     digit(80)
      integer*4     i,digix,tstdig,fdig
      real*4        fact10,realno
      logical*2     negnum

c External functions

      integer*4     FCHAR,LCHAR

c Calculate the positions of the first and last "characters" in
c the number

      fpos = FCHAR(chrnum)
      lpos = LCHAR(chrnum)

c Convert these characters into digits and store in a table (DIGIT)
c Check for presence of a negative sign while doing the pass

      digix = 0
      negnum = .false.

      do i = fpos,lpos

c        Check for a negative sign at the current character position

         curchr = chrnum(i:i)
         if (curchr .eq. negsgn) then
            negnum = .true.
         endif

c        Convert the character to a numeric digit

         ascii = ichar(chrnum(i:i))
         tstdig = ascii - asczer

c        Only record as a digit if it is a digit

         if (tstdig .ge. 0 .and. tstdig .le. 9) then

c        Record the position of the first digit

            if (digix .eq. 0) then
               fdig = i
            endif

c        Record the digit in table

            digix = digix + 1
            digit(digix) = tstdig

         endif

      enddo

c Find out what power of 10 the most signifigant digit of the
c number should be

c     Look for a possible decimal position in the string

      decpos = index(chrnum,deciml)
c      write (*,*) 'CHR_RL: decpos=',decpos

c     This decimal position is only valid if it is within the string
c     of digits

      if ((decpos .ge. fpos) .and. (decpos .le. lpos)) then

c     CASE1: a real number with a decimal point

         fact10 = 10**(decpos - fdig - 1)
c         write (*,*) 'CASE1: decimal,fact10=',fact10

      else

c     CASE2: an integer since there is no decimal point

         fact10 = 10.0**(digix-1)
c         write (*,*) 'CASE2: no decimal,fact10=',fact10

      endif

c Calculate the real number by multiplying each digit in the
c digit table by the appropriate power of ten and summing

      realno = 0.0
      do i = 1,digix
         realno = realno + fact10*digit(i)
         fact10 = fact10/10
c         write (*,*) 'i,digit(i)=',i,digit(i)
      enddo

c Return the converted character real number as a negative or
c positive real*4 number
 
      if (negnum) then
         CHR_RL = - realno
      else
         CHR_RL = realno
      endif

c      write (*,*) 'CHR_RL: final value=',chr_rl
      return
      END

c *******************************************************************
c FCHAR: A function which returns the first position of a
c ------ non-blank character in a string STRING
c        If the string is all blanks, then zero is returned in FCHAR
c *******************************************************************

      integer*4 function FCHAR(string)

c Constants

      include 'chardef.cns'

c Parameters

      character*(*) string

c Work variables

      integer*4 i,strlng
      logical*2 found

c Loop from the first character to the last checking each character
c for the first non-blank character

      found = .false.
      strlng = len(string)
      i = 1
      do while (.not. (found) .and. (i .le. strlng))
         if (string(i:i) .eq. BLANK) then
            i = i + 1
         else
            found = .true.
         endif
      enddo

c If a first character is not found then return zero otherwise
c return the position of the first character in the string

      if (found) then
          FCHAR = i
      else
          FCHAR = 0
      endif

      END

c *******************************************************************
c LCHAR: A function which returns the last position of a
c ------ non-blank character in a string STRING.
c        If the string is all blanks, then zero is returned in LCHAR
c *******************************************************************

      integer*4 function LCHAR(string)

c Constants

      include 'chardef.cns'

c Parameters

      character*(*) string

c Work variables

      integer*4 i,strlng
      logical*2 found

c Loop from the last character to the first checking each character
c for the first non-blank character

      found = .false.
      strlng = len(string)
      i = strlng
      do while (.not. (found) .and. (i .ge. 1))
         if (string(i:i) .eq. blank) then
            i = i - 1
         else
            found = .true.
         endif
      enddo

c If a last character is not found then return zero otherwise
c return the position of the last character in the string

      if (found) then
          LCHAR = i
      else
          LCHAR = 0
      endif
      return
      END

c *********************************************************************
c ENTREAL: A subroutine which gets a real number FINAL_REAL from the
c -------  user.
c          A default real may be specifed by setting DEFAULT_ON to
c          true and setting DEFAULT_REAL to the appropriate value.
c          A prompt may be specified by setting the string DESCR
c          accordingly.
c *********************************************************************

      subroutine ENTREAL(descr,default_on,default_real,
     +                  final_real,OK)

c Constants

      include 'chardef.cns'

c Parameters

      character*(*) descr
      logical       default_on,OK
      real*4        default_real
      real*4        final_real

c Work variables

      character*79  outstr
      character*32  intstr,tempstr
      integer*4     first_digit_pos,last_digit_pos

c External functions

      integer*4     FCHAR,LCHAR
      real*4        CHR_RL

c Create the prompt with default if desired

      outstr = descr
      if (default_on) then
         tempstr = BLANK
         write (tempstr,*) default_real
         first_digit_pos = FCHAR(tempstr)
         last_digit_pos  = LCHAR(tempstr)
         outstr = outstr(:LCHAR(outstr)) // ' [ ' //
     +            tempstr(first_digit_pos:last_digit_pos) // ' ]'
      endif
      outstr = outstr(:LCHAR(outstr)) // ': '

c Display the prompt and allow user to enter his/her choice
c [RETURN] = accept default

      write (*,1000) outstr(:LCHAR(outstr))
1000  format(1x,a,$)

      read (*,'(a)') intstr

c Process response

      if (intstr .eq. BLANK .and. default_on) then
         final_real = default_real
         OK = .true.
         return
      endif

      if (index(intstr,'.').eq.0) then
         intstr(lchar(intstr)+1:lchar(intstr)+1) = '.'
      end if

      decode (32,10,intstr,err=20) final_real
10    format (f32.5)
      OK = .true.
      return

20    final_real = default_real
      OK = .false.
      return

      end

