C**************************************************************************
C PLOTTER Routines
C
C DESCRIPTION:
C    These are the non-interactive routines that do the bulk of the
C    work for the PLOTTER program.
C
C COMPILATION:
C                    for plotrout
C
C AUTHOR: Nadia Shewchenko
C         Mark Tooker
C    
C         Aug. 1989
C
C**************************************************************************

C**************************************************************************
      subroutine Get_File_Size (UnitNum,file_type,nrows,ncols,first)
C                ~~~~~~~~~~~~~
C
C DESCRIPTION: 
C      Reads the input data file, to find out the number of 
C      rows & columns.
C
C INPUT PARAMETERS:
C      UnitNum - logical unit # of input file
C      file_type - format of input file: 1 - Morpholog; 2 - BSQ;
C                  3 - PIC; 4 - RNL
C      first - true if this is the first plot
C
C OUTPUT PARAMETERS:
C      nrows,ncols - number of rows, columns
C
C**************************************************************************

C........... subroutine parameters .......................

      integer   UnitNum,file_type
      integer*4 nrows,ncols
      logical   first,OK

C............ utility storage ............................

      integer*2 ncols2,nrows2

C**************************************************************************

      if (file_type.eq.3) then         ! PIC type
         read (UnitNum) ncols2, nrows2
         ncols = ncols2
         nrows = nrows2
                             ! closing is necessary so the entire file
	 close (UnitNum)     ! can be read from scratch by Decode_PIC later

      else if ((file_type.eq.1).or.(file_type.eq.2)) then    
C                                      ! morpholog or BSQ type
         read (UnitNum) nrows, ncols

      else if (file_type.eq.4) then    ! RNL type
100      print *,' '
         print *,'Enter file dimensions'
         if (first) then
            nrows = 512
            ncols = 512
         end if

110      call ENTINT ('   Number of Rows',.true.,nrows,nrows,OK)
            if (.not.OK) goto 110
120      call ENTINT ('   Number of Columns',.true.,ncols,ncols,OK)
            if (.not.OK) goto 120

         if ((ncols.lt.1).or.(ncols.gt.4096)) then
            print *,' '
            print *,'*** ERROR *** Number of columns not valid = ',
     +                ncols
            print *,' '
            goto 100
         end if
         if ((nrowss.lt.1).or.(nrows.gt.4096)) then
            print *,' '
            print *,'*** ERROR *** Number of rows not valid = ',nrows
            print *,' '
            goto 100
         end if
      end if                 

      return
      end

C*************************************************************************
      subroutine Read_Palette (OK,Is_RGB,PalName,zclass,kolor,
     +                         ic,im,iy,TextColor,BGColor,MaxColors)
C
C DESCRIPTION: 
C      Reads the input palette file, setting up ic,im,iy, and
C      kolor and zclass for later color initialization.
C
C INPUT PARAMETERS:
C      PalName - name of palette file
C      MaxColors - max. # of colors to read from palette
C      Is_RGB - true if this is RGB palette; false if CMY
C
C OUTPUT PARAMETERS:
C      OK - true if read OK
C      zclass, kolor - see comments in subroutine Set_Up_Color_Table
C      ic,im,iy - three-channel color values forming palette
C      TextColor - pixel value to be interpreted as text
C      BGColor - pixel value to be interpreted as background
C      
C**************************************************************************

C........... subroutine parameters .......................

      dimension kolor(99),zclass(98)
      integer ic(99),im(99),iy(99)
      integer*2 TextColor,BGColor,MaxColors
      logical OK,Is_RGB
      character*32 PalName

C**************************************************************************

      open(unit=40,file=PalName,form='formatted',
     +     access='sequential',status='old',err = 2)
      go to 3
2     print *,'**ERROR** Unable to open palette file ',PalName
      OK = .false.
      return

C...read the .PAL file and update the GIMAGE color table..........
          
3     print *,' '

      do jj = 1,MaxColors
         read(40,*,end=88),i,ic(jj),im(jj),iy(jj)
      end do
88    continue
      close(40)

C.....set up black (text) entry.....

      TextColor = 99

      if (Is_RGB) then
         ic(TextColor) = 0     ! RGB black
         im(TextColor) = 0
	 iy(TextColor) = 0
      else
         ic(TextColor) = 255       ! CMY black
         im(TextColor) = 255
         iy(TextColor) = 255
      end if

C.....set up white (background) entry.....
          
      BGColor = 97

      if (Is_RGB) then
         ic(BGColor) = 255     ! RGB white
         im(BGColor) = 255
	 iy(BGColor) = 255
      else
         ic(BGColor) = 0       ! CMY black
         im(BGColor) = 0
         iy(BGColor) = 0
      end if
          
C..........set up class definitions & limits for GIMCOL & GIMZCL.........

C...the simplest case is used, where Z- (ie. pixel) values of under 1.0
C...(ie. background = 0) are associated with color entry 1, 1.0 - 2.0
C...with entry 2, etc.................................................

      do i = 1,98
         zclass(i) = FLOATJ (i-1)
         kolor(i)  = i
      end do
      kolor(99) = 99

      OK = .true.

      return
      end

C *****************************************************************
C    SUBROUTINE : Set_Up_Color_Table
C
C    PURPOSE: This routine updates the UNIRAS colour table,
C             using data derived from Set_up_Color_Table_Queries.
C
C    INPUT PARAMETERS:
C             patterns  - true if B&W colour scheme used
C	      Is_RGB    - true if RGB to be used; false if CMY
C             kolor     - (see comments in code below)
C             zclass    -   "    "     "     "     "
C             ic,im,iy  - arrays containing palette color values
C             NewTable  - true if new color table to be loaded;
C                         false if standard palette to be used
C
C ****************************************************************

      subroutine Set_Up_Color_Table (patterns,NewTable,Is_RGB,
     +                               kolor,zclass,ic,im,iy)

      dimension kolor(99),zclass(98)
      integer ic(99),im(99),iy(99)
      character palette*1,pal*50,fin*50,bwpal*50

      character BicName*50, PalName*50, More

      logical patterns, Is_RGB, NewTable

C.....Color Plots.............................................................

      if (.not.patterns) then

         if (NewTable) then

C...........update the UNIRAS color table, starting at entry 2......

            call gcolor (2,ic,im,iy,99)
         end if

C.....black and white plots - not working.....
      
      else
         call gpatt
         call gcomap
C        call gcolor (20,30,0,0,1)
      end if
   
C ------------------------------------------------------------------
C GIMZCL(zclass,ncl) - defines class limits to be used by GIMAGE
C                      to display an image. The intervals correspond
C                      to the levels of the color scale as defined
C                      by GIMCOL
C
C                      zclass - a user declared array containing the 
C                               class limits which define the intervals.
C		       ncl    - the number of class limits (= number of
C                               classes minus one)
C
C GIMCOL(kolor,ncol) - defines a color scale to be used by GIMAGE to
C                      display an image. Each level of the color scale
C                      corresponds to an interval of data values as
C                      defined by GIMZCL.
C
C                      kolor - a 1-dim user declared array of 
C                              colour indices
C                      ncol  - number of color indices (dim of kolor)
C
C ------------------------------------------------------------------

      call gimzcl(zclass,98)	
      call gimcol(kolor,99)

      return
      end

C *****************************************************************
C    SUBROUTINE : Initialize_Graphics
C
C    PURPOSE: This routine calls all the needed initialization
C             routines for UNIRAS/GIMAGE, including device selection.
C
C    INPUT PARAMETERS:
C             plotter   - the choosen plotter
C             lastplotter - plotter used last time
C             plpix     - image output grid size in mm
C             xorig     - the origin in the x direction in mm
C             yorig     - the origin in the y direction in mm
C
C ****************************************************************

      subroutine Initialize_Graphics(plotter,lastplotter,
     +                               plpix,xorig,yorig)

      integer plotter,lastplotter
      real*4 plpix,xorig,yorig

      print *,' '
      print *, 'SELECTING PLOTTER....'
      print *,' '
      if (plotter.eq.1) then
         call groute('select GAPPL;exit')       ! Applicon
         print *,' GAPPL device selected.'
      else if (plotter.eq.2) then
         call groute('select GVCE3436;exit')    ! Versatec
         print *,'GVCE3436 device selected.'
      else if (plotter.eq.3) then
         call groute('select GOPTC;exit')       ! Optronics
         print *,'GOPTC device selected.'
      else if (plotter.eq.4) then
         call groute('select MT41XX;exit')      ! Tektronix 41**
      else
         call groute ('help;exit')
         call groute(' ') 			! let the user select a device
5        print 10                               ! and pixel size
10       format (/,' Enter pixel size in mm: ',$)
         read (*,*,err=5) plpix
         if (plpix.le.0.0) then
            print *, '*** INVALID *** Cannot be equal to or less than 0'
            goto 5
         endif
      endif

      if (plotter.ne.4) then
         print *,' '
         print *,'SETTING UP GIMAGE....'
      end if

C      call gshmes('DIS','DIS')	  ! display short message for each uniras c call
      call gexcol		  ! extend the colour table to 5000 entries
      call gwrksz(176000)	  ! adjust work space size.
C      call gwrksz(352000)	  ! adjust work space size.

      if (lastplotter.ne.plotter) then
        call gopen			! initialize device
      else
        call gropen
      endif

C.....set other parameters....................................................

C..........................................................
C......grota works for the rrect, titles, etc. but not.....
C......for the plot itself, which GIMAGE stubbornly.....
C......draws column by column no matter what else goes on.....
C      if (plotter.eq.4) then
C         call grota                     ! rotate 90 degrees on tektr. screen
C      end if
C..........................................................

      call gimgrd(plpix,plpix)		! set the grid size in mm, x and y dir.	

      call ghalf8		        ! select tone shading (Must for Vers.)

      call gimori(xorig,yorig)		! set the origiin

      return
      end

C *****************************************************************
C    SUBROUTINE : Set_Color_Mode
C
C    PURPOSE: This routine sets up GIMAGE to either B & W, RGB,
C             or CMY color mode.
C
C    INPUT PARAMETERS:
C             patterns - true if B&W; false if color
C             Is_RGB   - true if RGB; false if CMY
C
C ****************************************************************

      subroutine Set_Color_Mode (patterns,Is_RGB)

      logical patterns, Is_RGB

      integer*4 RGB_Mode, CMY_mode, BW_Mode

      data BW_mode  /7/     ! GIMAGE values for setting up plot
      data RGB_mode /1/     ! - see p. 33 GIMAGE manual 
      data CMY_mode /2/     !
   
C.....initialize the color mode................................................

      if (patterns) then
C         call gcmode(-(BW_mode),iamode)   ! set colour mode B/W subtractive
         call rcmode('BWS',100)

      else 

	 if (Is_RGB) then
            call gcmode(-(RGB_mode),iamode)	! see p.33, GIMAGE manual
	    call rcmode ('RGB',255)
	 else
            call gcmode(-(CMY_mode),iamode)	
	    call rcmode ('CMY',255)
         end if
      end if

      return
      end

C *****************************************************************
C    SUBROUTINE : Set_Up_Channels
C
C    PURPOSE: This routine sets up GIMAGE to read in the 3 input
C             channels
C
C    INPUT PARAMETERS:
C             rmax - color intensity range
C
C ****************************************************************

      subroutine Set_Up_Channels (rmax)

      real*4 rmax

      do i = 1,3
         call gimchl(i,0.0,rmax,16,i,3)
      end do

      return
      end

C***********************************************************************
C    SUBROUTINE : Plot_Class_File
C
C    PURPOSE: This routine uses GIMAGE to write the image to the plot
C             file 'UNIRAST'. This routine is
C             used when the data consists of colour class numbers. If
C             the data is organized in 3 channels Plot_Channel_Files
C             is used instead.
C
C    INPUT PARAMETERS:
C             ncols     - the number of pixels per line in the input image
C             nrows     - the number of scan lines in the input image
C             plotter   - the choosen plotter
C             plpix     - output pixel size in mm
C             fin       - file name of data file
C             file_type - format of data file
C             TextColor - value to which class 255 should be changed 
C             BGColor   - value to which class 0 (background) should
C                         be changed
C             xorig,
C             yorig     - x-y origin of plot, in mm
C             Shorten   - true if abbreviated plot wanted
C             ShortNum  - number of lines to plot before quitting
C
C *************************************************************************

      subroutine Plot_Class_File(ncols,nrows,plotter,plpix,fin,
     +                           file_type,TextColor,BGColor,
     +                           xorig,yorig,Shorten,ShortNum)

      real z1(2048)
      integer ncols,nrows,newcols
      integer plotter,file_type
      integer*2 read_im(4096,4096), TextColor, BGColor
      character read_im1(4096)
      real*4 plpix,xorig,yorig
      character fin*50
      logical OK, bad_data,Shorten
      integer*4 ShortNum

C*****************************************************************************

        newcols = ncols

C.......Read in the entire data file.......................................

        if (file_type.eq.3) then

C..........read from run-length encoded .PIC file.....

           if (plotter.ne.4) then
              print *, ' '
              print *, 'Decoding file ', fin
           end if

           call Decode_PIC (OK, fin, nCols, nRows, read_im)
           if (.not.OK) then
              goto 990
           end if

        else if (file_type.eq.1) then

C..........read from morpholog-type file.....

           if (plotter.ne.4) then
              print *, ' '
              print *, 'Reading file ', fin
           end if

           do ix = 1,nrows
	      read (1,end=999) (read_im(iy,ix), iy = 1,ncols)
           end do

        else if (file_type.eq.2) then

C..........read from BSQ type file.....

           if (plotter.ne.4) then
              print *, ' '
              print *, 'Reading file ', fin
           end if

           numrecs = jint (ncols/512.0)    ! determine how many 512-byte
           if (jmod(ncols,512).gt.0) then  ! records to read
              numrecs = numrecs + 1
           end if

           do ix = 1,nrows

C.............read 512-byte records until end of row.....

              iy = 1
              do irec = 1,numrecs
                 iyend = iy + 511
                 read (1,end=999) (read_im1(iy), iy=iy,iyend)
              end do

C.............convert entire row to 2-byte integers.....

              do iyy = 1,ncols
                 read_im(iyy,ix) = ichar(read_im1(iyy))
              end do

           end do

        else

C..........read from RNL type file.....

           if (plotter.ne.4) then
              print *, ' '
              print *, 'Reading file ', fin
           end if

           do ix = 1,nrows

              read (1,end=999) (read_im1(iy), iy = 1,ncols) ! read 1-byte data

              do iyy = 1,ncols                              ! convert to 2-bytes
                 read_im(iyy,ix) = ichar(read_im1(iyy))
              end do

          end do


	end if

C.......finished reading data file.....

C.......shorten plot if so desired.....

        if ((Shorten).and.(ShortNum.lt.nrows)) then
           nrows = ShortNum
        end if

        if (plotter.ne.4) then
           print *,' '
           print *,'Must plot ',nrows,' lines'
           print *,' '
       
        else

C.....draw a rectangle around plot area on tektronix screen.....

           call rrect(xorig,yorig,xorig+nrows*plpix,
     +                yorig+ncols*plpix,0,1.)

        end if

        call gimage(z1,newcols,0)	   		! gimage initialization

        bad_data = .false.

C.......loop for every row in output image.................................

	do iy = 1,nrows

           if (plotter.ne.4) then
              print 1,iy
1             format ('+ Plotting line ',i4)
           end if

C..........loop for every column in output image...........................

	   do ix = 1,ncols

C.............convert integer values to real.....

              z1(ix) = read_im(ix,iy)

C.............check for bad data - should be only 1-byte unsigned values...

	      if ((z1(ix).gt.255.).or.(z1(ix).lt.0.)) then
                 if (plotter.ne.4) then
                    print *,' '
                    print *,'*** Error *** Bad input data = ',z1(ix),
     +		            ' in line:', iy, ', column:', ix
                 else
                    bad_data = .true.
                 end if
                 z1(ix) = 0.0
 	      endif

C.............Since there can be a maximum of 99 classes in GIMAGE, input
C.............text pixel values of 255  must be changed to the predetermined 
C.............TextColor value (black).
C.............Similarly, values of 0 (background) or 250-254 (dots) must be 
C.............changed to BGColor (white).

              if (z1(ix).eq.255.) then
		 z1(ix) = TextColor
              else if ((z1(ix).eq.0.0).or.(z1(ix).ge.250.)) then
                 z1(ix) = BGColor
	      end if

	   end do

C..........plot the entire line.....

           call gimage(z1,newcols,10)

        end do
      
23      call gimage(z1,newcols,9999)		! gimage termination

        if (bad_data) then
           print *, 'Bad data (< 0 or > 255) was found.'
        end if

        goto 1000

990   continue
      print *,' '
      print *,'*** ERROR *** decoding file ', fin
      print *,'No plotting done.'
      goto 1000
   
999   continue
      print *,' '
      print *,'*** ERROR *** reading file ', fin
      print *, 'row = ',ix,' column = ',iy
      print *,'No plotting done.'
      goto 1000

1000  continue

      call gclose 		! terminate current picture generation

      close(1)

      return
      end


C *****************************************************************
C    SUBROUTINE : Plot_Channel_Files
C
C    PURPOSE: This routine uses GIMAGE to write the image to the plot
C             file 'UNIRAST'. This routine is
C             used when the data  is organized in 3 channels. If data
C             consists of colour class numbers Plot_Class_File is used
C             instead.
C
C    INPUT PARAMETERS:
C             plotter - the choosen plotter
C             iysiz   - the num. of lines in the output image
C             ixsiz   - the num. of pixels per line in the output image
C             nrows   - the number of scan lines in the input image
C             ncols   - the number of pixels per line in the input image
C             nchan   - the number of channels
C             nlook   - 1 if color plot; 3 if b/w plot   *** NOT USED
C             looks   - true if lookup tables exist      *** NOT USED
C             lut     - lookup table array               *** NOT USED
C             plpix   - pixel size - used to draw rectangle
C             fin     - array of 3 file names, 1 per channel
C             file_type - format of data file
C             xorig,
C             yorig     - x-y origin of plot, in mm
C             Shorten   - true if abbreviated plot wanted
C             ShortNum  - number of lines to plot before quitting
C
C ****************************************************************

      subroutine Plot_Channel_Files(plotter,iysiz,ixsiz,nrows,
     +                          ncols,nchan,nlook,looks,lut,plpix,
     +                          fin,file_type,xorig,yorig,
     +                          Shorten,ShortNum)

      integer plotter,file_type
      integer*4 iysiz,ixsiz,nrows,ncols,nchan,nlook,
     +          lut(3,0:255),ShortNum
      character*50 fin(3)
      real*4 plpix,xorig,yorig
      logical looks,Shorten
      logical Bad_Data

      integer*2 read_word (4096,3)
      integer*4 numrecs

      character read_char (4096,3)
      logical*1 read_byte (4096,3)
      equivalence (read_char,read_byte)

      dimension z (3,4096)

C*****************************************************************************

C.......shorten plot if so desired.....

        if ((Shorten).and.(ShortNum.lt.nrows)) then
           nrows = ShortNum
        end if

        if (plotter.ne.4) then
           print *,' '
           print *,'Must plot ',nrows,' lines'
           print *,' '

        else

C..........draw a rectangle around plot area on tektronix screen.....

           call rrect(xorig,yorig,xorig+nrows*plpix,
     +                yorig+ncols*plpix,0,1.)

        end if

        call gimage(z,ncols,0)	   		! gimage initialization

        bad_data = .false.

        if (file_type.eq.2) then
           numrecs = jint (ncols/512.0)
           if (jmod(ncols,512).gt.0) then
              numrecs = numrecs + 1
           end if
        end if

C.......loop for every row in input image.................................

        do ix = 1,nrows       ! remember x is vertical axis, y horizontal

           if (plotter.ne.4) then
              print 1,ix
1             format ('+ Plotting line ',i4)
           end if

C..........read in the data from channel files.....

           if (file_type.eq.2) then

C.............read from BSQ files.....

              ifile = 1
              iy = 1
              do irec = 1,numrecs
                 iyend = iy + 511
                 read (1,end=990) (read_byte(iy,1), iy=iy,iyend)       ! file 1
              end do
              if (nchan.gt.1) then
                 ifile = 2
                 iy = 1
                 do irec = 1,numrecs
                    iyend = iy + 511
                    read (2,end=990) (read_byte(iy,2), iy=iy,iyend)     ! file 2
                 end do
                 if (nchan.gt.2) then
                    ifile = 3
                    iy = 1
                    do irec = 1,numrecs
                       iyend = iy + 511
                       read (3,end=990) (read_byte(iy,3), iy=iy,iyend)  ! file 3
                    end do
                 else
                    do iy = 1,ncols
                       read_byte(iy,3) = read_byte(iy,1)   ! case only 2 files
                    end do
                 end if
              else
                 do iy = 1,ncols
                    read_byte(iy,2) = read_byte(iy,1)      ! case only 1 file
                    read_byte(iy,3) = read_byte(iy,1)
                 end do
              end if

           else ! (file_type.eq.1)   Morpholog file

              ifile = 1
	      read (1,end=999) (read_word(iy,1), iy = 1,ncols)  ! file 1
              if (nchan.gt.1) then
                 ifile = 2
                 read (2,end=999) (read_word(iy,2), iy = 1,ncols)  ! file 2
                 if (nchan.gt.2) then
                    ifile = 3
                    read (3,end=999) (read_word(iy,3), iy = 1,ncols)  ! file3
                 else
                    do iy = 1,ncols
                       read_word(iy,3) = read_word(iy,1)   ! case only 2 files
                    end do
                 end if
              else
                 do iy = 1,ncols
                    read_word(iy,2) = read_word(iy,1)      ! case only 1 file
                    read_word(iy,3) = read_word(iy,1)
                 end do
              end if

           end if 

C..........loop for every column in output row.....

	   do iy = 1,ncols     

C.............loop for each channel.....

              do ic = 1,3
                 il=ic
                 if (nchan.eq.1.and.nlook.eq.1) il=1
                 icc=ic
                 if (ic.gt.nchan) then
                    icc=1
                 end if

                 if (file_type.eq.2) then
                    idum = ichar(read_char(iy,icc))  ! BSQ has 1-byte pixels
                 else ! (file_type.eq.1)
                    idum = read_word(iy,icc)         ! Morph.  2-byte "   "
                 end if

C.......................................................................
C                 if (looks) then
C                    z(ic,iy) = lut(il,idum)
C                 else
                     z(ic,iy) = idum
C                 endif
C.......................................................................

                 if (z(ic,iy).gt.255..or.z(ic,iy).lt.0.) then
                    if (plotter.ne.4) then
                       print *,' '
                       print *,'*** Error *** Bad input data = ',
     +                         z(ic,iy),
     +	                       ' in row:', ix, ', column:', iy
                       print *,'Channel = ',ic
                    else
                       bad_data = .true.
                    end if
                    z(ic,iy)=0.
                 endif

              end do  ! ic = 1,3

           end do  ! iy = 1,ncols

C..........plot the entire line.....

           call gimage(z,ncols,10)

        end do  ! ix = 1,nrows
      
23      call gimage(z,ncols,9999)		! gimage termination

        if (bad_data) then
           print *, 'Bad data (< 0 or > 255) was found.'
        end if

        goto 1000 

990   continue
      print *,' '
      print *,'*** ERROR *** decoding file ', fin(ifile)
      print *, 'Row =',ix, ' Column =',iy
      print *,'No plotting done.'
      goto 1000

999   continue
      print *,' '
      print *,'*** ERROR *** reading file ', fin(ifile)
      print *, 'Row =',ix,' Column =',iy
      print *,'No plotting done.'
      goto 1000

1000  continue

      call gclose 		! terminate current picture generation

      do i = 1,nchan
         close(i)
      end do

      return
      end

